@@ -2,16 +2,16 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 3)
+__version__ = (0, 0, 4)
 __all__ = [
     "as_thread", "ensure_async", "ensure_await", "ensure_coroutine", "ensure_aiter", 
     "async_map", "async_filter", "async_reduce", "async_zip", "async_chain", 
-    "call_as_aiter", "to_list", 
+    "async_all", "async_any", "call_as_aiter", "to_list", 
 ]
 
 from asyncio import to_thread
-from collections.abc import Awaitable, AsyncIterable, AsyncIterator, Callable, Coroutine, Iterable
-from inspect import isawaitable, iscoroutinefunction, isgenerator
+from collections.abc import Awaitable, AsyncIterable, AsyncIterator, Callable, Coroutine, Iterable, Iterator
+from inspect import isawaitable, iscoroutine, iscoroutinefunction, isgenerator
 from typing import cast, Any, ParamSpec, TypeVar
 
 from decotools import decorated
@@ -23,7 +23,12 @@ T = TypeVar("T")
 
 
 @decorated
-def as_thread(func: Callable[Args, T], /, *args, **kwds) -> Awaitable[T]:
+def as_thread(
+    func: Callable[Args, T], 
+    /, 
+    *args: Args.args, 
+    **kwds: Args.kwargs, 
+) -> Awaitable[T]:
     def wrapfunc(*args, **kwds):
         try:
             return func(*args, **kwds)
@@ -35,7 +40,7 @@ def as_thread(func: Callable[Args, T], /, *args, **kwds) -> Awaitable[T]:
 def ensure_async(
     func: Callable[Args, T | Awaitable[T]], 
     /, 
-    threaded: bool = True, 
+    threaded: bool = False, 
 ) -> Callable[Args, Awaitable[T]]:
     if iscoroutinefunction(func):
         return func
@@ -71,6 +76,8 @@ def ensure_await(o, /) -> Awaitable:
 
 
 def ensure_coroutine(o, /) -> Coroutine:
+    if iscoroutine(o):
+        return o
     async def wrapper():
         if isawaitable(o):
             return await o
@@ -81,7 +88,7 @@ def ensure_coroutine(o, /) -> Coroutine:
 def ensure_aiter(
     it: Iterable[T] | AsyncIterable[T], 
     /, 
-    threaded: bool = True, 
+    threaded: bool = False, 
 ) -> AsyncIterator[T]:
     if isinstance(it, AsyncIterable):
         return aiter(it)
@@ -120,24 +127,41 @@ def ensure_aiter(
     return wrapper()
 
 
-async def async_map(func, iterable, /, *iterables, threaded: bool = True):
-    func = ensure_async(func, threaded=threaded)
+async def async_map(
+    func: Callable[..., T], 
+    iterable: Iterable | AsyncIterable, 
+    /, 
+    *iterables: Iterable | AsyncIterable, 
+    threaded: bool = False, 
+) -> AsyncIterator[T]:
+    fn = ensure_async(func, threaded=threaded)
     if iterables:
         async for args in async_zip(iterable, *iterables, threaded=threaded):
-            yield await func(*args)
+            yield await fn(*args)
     else:
         async for arg in ensure_aiter(iterable, threaded=threaded):
-            yield await func(arg)
+            yield await fn(arg)
 
 
-async def async_filter(func, iterable, /, threaded: bool = True):
-    func = ensure_async(func, threaded=threaded)
+async def async_filter(
+    func: Callable[[T], bool], 
+    iterable: Iterable[T] | AsyncIterable[T], 
+    /, 
+    threaded: bool = False, 
+) -> AsyncIterator[T]:
+    fn = ensure_async(func, threaded=threaded)
     async for arg in ensure_aiter(iterable, threaded=threaded):
-        if (await func(arg)):
+        if (await fn(arg)):
             yield arg
 
 
-async def async_reduce(func, iterable, initial=undefined, /, threaded: bool = True):
+async def async_reduce(
+    func: Callable, 
+    iterable, 
+    initial=undefined, 
+    /, 
+    threaded: bool = False, 
+) -> AsyncIterator:
     ait = ensure_aiter(iterable, threaded=threaded)
     if initial is undefined:
         try:
@@ -151,7 +175,12 @@ async def async_reduce(func, iterable, initial=undefined, /, threaded: bool = Tr
     return prev
 
 
-async def async_zip(iterable, /, *iterables, threaded: bool = True):
+async def async_zip(
+    iterable, 
+    /, 
+    *iterables, 
+    threaded: bool = False, 
+) -> AsyncIterator:
     iterable = ensure_aiter(iterable, threaded=threaded)
     if iterables:
         fs = (iterable.__anext__, *(ensure_aiter(it, threaded=threaded).__anext__ for it in iterables))
@@ -165,13 +194,20 @@ async def async_zip(iterable, /, *iterables, threaded: bool = True):
             yield e
 
 
-async def async_chain(*iterables, threaded: bool = True):
+async def async_chain(
+    *iterables, 
+    threaded: bool = False, 
+) -> AsyncIterator:
     for it in iterables:
         async for e in ensure_aiter(it, threaded=threaded):
             yield e
 
 
-async def async_chain_from_iterable(iterable, /, threaded: bool = True):
+async def async_chain_from_iterable(
+    iterable, 
+    /, 
+    threaded: bool = False, 
+) -> AsyncIterator:
     async for it in ensure_aiter(iterable, threaded=False):
         async for e in ensure_aiter(it, threaded=threaded):
             yield e
@@ -179,11 +215,33 @@ async def async_chain_from_iterable(iterable, /, threaded: bool = True):
 setattr(async_chain, "from_iterable", async_chain_from_iterable)
 
 
+async def async_all(
+    iterable, 
+    /, 
+    threaded: bool = False, 
+) -> bool:
+    async for e in ensure_aiter(iterable, threaded=threaded):
+        if not e:
+            return False
+    return True
+
+
+async def async_any(
+    iterable, 
+    /, 
+    threaded: bool = False, 
+) -> bool:
+    async for e in ensure_aiter(iterable, threaded=threaded):
+        if e:
+            return True
+    return False
+
+
 async def call_as_aiter(
     func: Callable[[], T] | Callable[[], Awaitable[T]], 
     /, 
     sentinel = undefined, 
-    threaded: bool = True, 
+    threaded: bool = False, 
 ) -> AsyncIterator[T]:
     func = ensure_async(func, threaded=threaded)
     try:
@@ -202,6 +260,10 @@ async def call_as_aiter(
         pass
 
 
-async def to_list(it: Iterable[T] | AsyncIterable[T], /) -> list[T]:
-    return [e async for e in ensure_aiter(it)]
+async def to_list(
+    it: Iterable[T] | AsyncIterable[T], 
+    /, 
+    threaded: bool = False, 
+) -> list[T]:
+    return [e async for e in ensure_aiter(it, threaded=threaded)]
 
